<?php

namespace FiloBlu\Maintenance\Observer;

use Magento\Framework\Event\ObserverInterface;

class RedirectResponseBefore implements ObserverInterface {

    protected $_helper;
    protected $storeManager;

    public function __construct(
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \FiloBlu\Maintenance\Helper\Data $helper
    ) {
        $this->_helper = $helper;
        $this->storeManager = $storeManager;
    }

    public function execute(\Magento\Framework\Event\Observer $observer) {

        $request = $observer->getRequest();
        $response = $observer->getResponse();
        $path = trim($request->getOriginalPathInfo(), '/');

        
        if ($this->_helper->getConfig("general/enabled")) {
            $allowed_urls = $this->_helper->allowedUrls();
            $isAllowedUrl = ($path == "maintenance");

            if(!$isAllowedUrl){
                foreach($allowed_urls as  $url) {
                    $url = trim($url , '/');
                    if($url == $path){
                        $isAllowedUrl = true;
                        return;
                    }
                }
            }


            if ($this->_helper->getConfig("full_page_cache/caching_application", "system") == \Magento\PageCache\Model\Config::VARNISH) {
                $response->setNoCacheHeaders();
            }

            if (!$this->_helper->isAllowedIp() && !$this->_helper->isAdminOnlyAllowed() && !$isAllowedUrl) {

                $response->setRedirect($this->storeManager->getStore()->getBaseUrl() . "maintenance");
                $response->sendResponse();
               
            }

            if ($this->_helper->isAdminOnlyAllowed() && !$this->_helper->isValidAdmin() && !$isAllowedUrl) {

                $response->setRedirect($this->storeManager->getStore()->getBaseUrl() . "maintenance");
                $response->sendResponse();
                   
            }
        } else {

            if ($path == "maintenance") {

                if ($this->_helper->getConfig("full_page_cache/caching_application", "system") == \Magento\PageCache\Model\Config::VARNISH) {
                    $response->setNoCacheHeaders();
                }


                if (!$this->_helper->isAllowedIp()) {
                    $response->setRedirect($this->storeManager->getStore()->getBaseUrl());
                    $response->sendResponse();
                }
            }
        }
    }

}
